package com.contec.jar.cms50ew;


import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;

public class DeviceCommand {

	/** Instruction packet */
	public static final int e_pack_command = 0x7D;// Request Command
	/**
	 * request user info of device
	 * @return byte array of request user info command
	 */
	public static byte[] REQUEST_USER_INFO(){
		byte[] _commd = {
				(byte) e_pack_command,
				(byte) 0x80,// Request user info
				(byte) 0xAB, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return _commd;
	}
	
	/**
	 * request version info of device
	 * @return byte array of request version info command
	 */
	public static byte[] REQUEST_VERSION_INFO(){
		byte[] _commd = {
				(byte) e_pack_command,
				(byte) 0x80,// Request user info
				(byte) 0xA0, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		DevicePackManager.doPack(_commd);
		return _commd;
	}
	/**
	 * request whether have pi
	 * @return byte array command of whether have pi
	 */
	public static byte[] REQUEST_PI_TYPE(){
		byte[] _commd =  {
				(byte) e_pack_command,
				(byte) 0x80, // Request if has PI
				(byte) 0xAC, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return _commd;
	}

	/**
	 * request the count of storage data
	 * @return byte array of request data count command
	 */
	public static byte[] REQUEST_DATA_COUNT(){
		byte[] _commd = {
				(byte) e_pack_command,
				(byte) 0x80,// Request data count
				(byte) 0xA3, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return _commd;
	}
	
	/**
	 * request the data of measure time 
	 * @param position the data index 
	 * @return byte array of request the data measure time command
	 */
	public static byte[] REQUEST_DATA_DATE(int position) {
		byte comdt[] = { (byte) e_pack_command, (byte) 0x80, (byte) 0xA5,
				(byte) 0x00, (byte) position, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return comdt;
	}

	/**
	 * 
	 * @param position the data of index
	 * @return byte array of request data length command
	 */
	public static byte[] REQUEST_DATA_LEN(int position) {
		byte comdd[] = { (byte) e_pack_command, (byte) 0x80, (byte) 0xA4,
				(byte) 0x00, (byte) position, (byte) 0x00, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return comdd;

	}
	
	/**
	 * 
	 * @param position the data index
	 * @param _PI pi type ,if _PI is false,the position is no use
	 * @return byte array of request data command
	 */
	public static byte[] REQUEST_DATA(int position, boolean _PI) {

		byte _comddata[];
		if (_PI) {
			byte comddata[] = { (byte) e_pack_command, (byte) 0x80,
					(byte) 0xA6, (byte) 0x00, (byte) position, (byte) 0x00,
					(byte) 0x00, (byte) 0x00, (byte) 0x00 };

			_comddata = comddata;
		} else {
			byte comddata[] = { (byte) e_pack_command, (byte) 0x81,
					(byte) 0xA6, (byte) 0xFF, (byte) 0xFF };

			_comddata = comddata;
		}
		return _comddata;
	}

	/**
	 * 
	 * @param position if all is true ,position is no use,else position is data index
	 * @param all whether delete all data
	 * @return byte array of delete data command
	 */
	public static byte[] DELETE_DATA(int position,boolean all){
		byte _comdDelete[];
		if(all){
			byte comddata[] = { (byte) e_pack_command, (byte) 0x81,
					(byte) 0xAE, (byte) 0xFF, (byte) 0xFF };

			_comdDelete = comddata;
		}else{
			byte comddata[] = { (byte) e_pack_command, (byte) 0x80,
				(byte) 0xAE, (byte) 0x00, (byte) position, (byte) 0x00,
				(byte) 0x00, (byte) 0x00, (byte) 0x00 };

			_comdDelete = comddata;
		}
		return _comdDelete;
	}
	public static byte[] BACK_COMMAND(int position) {
		byte comd[] = { (byte) DeviceCommand.e_pack_command, (byte) 0x80,
				(byte) 0xB9, (byte) 0, (byte) (position), (byte) 0x00,
				(byte) 0x00, (byte) 0x00, (byte) 0x00 };
		return comd;
	}

	/**
	 * set date of ew 
	 * @return byte array of set date command
	 */
	public static byte[] SET_DATE() {

//		int mYear ;
//		int mMonth;
//		int mDay;
//		int mWeek = Calendar.getInstance().get(Calendar.DAY_OF_WEEK) - 1;
//		 SntpClient client = new SntpClient();  
//	        if (client.requestTime("pool.ntp.org", 30000)) {  
//	            long now = client.getNtpTime() + System.nanoTime() / 1000  
//	                    - client.getNtpTimeReference();  
//	           Date current = new Date(now);  
//	           mYear = current.getYear()-100;
//	           mMonth = current.getMonth()+1;
//	           mDay = current.getDate();
//	            System.out.println(current.toString());  
//	        }else{
//	    		mYear = Calendar.getInstance().get(Calendar.YEAR) - 2000;
//	    		mMonth = Calendar.getInstance().get(Calendar.MONTH) + 1;
//	    		mDay = Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
//	        }
		int mYear = Calendar.getInstance().get(Calendar.YEAR) - 2000;
		int mMonth = Calendar.getInstance().get(Calendar.MONTH) + 1;
		int mDay = Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
		int mWeek = Calendar.getInstance().get(Calendar.DAY_OF_WEEK) - 1;
		byte[] SET_DATE = { (byte) 0x7D, (byte) 0x80, (byte) 0xB2, (byte) 0x20,
				(byte) mYear, (byte) mMonth, (byte) mDay, (byte) mWeek,
				(byte) 0x00 };
		DevicePackManager.doPack(SET_DATE);
		return SET_DATE;
	}

	/**
	 * set time of ew 
	 * @return byte array of set time command
	 */
	public static byte[] SET_TIME() {

		
//		int mHours ;
//		int mMinutes;
//		int mSeconds;
//		int mWeek = Calendar.getInstance().get(Calendar.DAY_OF_WEEK) - 1;
//		 SntpClient client = new SntpClient();  
//	        if (client.requestTime("pool.ntp.org", 30000)) {  
//	            long now = client.getNtpTime() + System.nanoTime() / 1000  
//	                    - client.getNtpTimeReference();  
//	           Date current = new Date(now);  
//	           mHours = current.getHours();
//	           mMinutes = current.getMinutes();
//	           mSeconds = current.getSeconds();
//	            System.out.println(current.toString());  
//	        }else{
//	    		mHours = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
//	    		mMinutes = Calendar.getInstance().get(Calendar.MINUTE);
//	    		mSeconds = Calendar.getInstance().get(Calendar.SECOND);
//	        }
		int mHours = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		int mMinutes = Calendar.getInstance().get(Calendar.MINUTE);
		int mSeconds = Calendar.getInstance().get(Calendar.SECOND);

		byte[] SET_TIME = { (byte) 0x7D, (byte) 0x80, (byte) 0xB1,
				(byte) mHours, (byte) mMinutes, (byte) mSeconds, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		DevicePackManager.doPack(SET_TIME);
		return SET_TIME;
	}
	/**
	 * 设备握手协议
	 */
	public static byte[] device_Handshake_new(byte[] data){
		byte[] code = new byte[8];
		byte[] md5_code;
		try {
			md5_code = MessageDigest.getInstance("MD5").digest(data);
			
			for (int i=0; i<8; ++i)
			{
				code[i] = (byte)((md5_code[i] ^ md5_code[i + 8]) | 0x80);
			}
			

			byte[] handshake = new byte[]{(byte) 0xBF,(byte) 0x00,(byte) 0x00,(byte) 0x00,(byte) 0x00,
					(byte) 0x00,(byte) 0x00,(byte) 0x00,(byte) 0x00,(byte) 0x00};
			
			return sum_Check1(handshake);
		} catch (NoSuchAlgorithmException e) {
			
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * 查询设备版本号
	 */
	public static byte[] set_CompareVersion(){
		byte[] compare = new byte[]{(byte) 0x82,(byte) 0x00};
		return sum_Check1(compare);
	}
	
	/**
	 * 对时
	 */
	public static byte[] set_time_new(){
		
		int mYear = Calendar.getInstance().get(Calendar.YEAR) - 2000;
		int mMonth = Calendar.getInstance().get(Calendar.MONTH) + 1;
		int mDay = Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
		
		int mHours = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		int mMinutes = Calendar.getInstance().get(Calendar.MINUTE);
		int mSeconds = Calendar.getInstance().get(Calendar.SECOND);

		byte[] set_time = { (byte) 0x83, (byte) mYear, (byte) mMonth,(byte) mDay,
				(byte) mHours, (byte) mMinutes, (byte) mSeconds, (byte) 0x00,
				(byte) 0x00, (byte) 0x00 };
		return sum_Check1(set_time);
	}
	
	/**
	 * 查询连续数据
	 * @param pType
	 * 		0：脉率，1：血氧
	 * @return
	 */
	public static byte[] query_Continuous_Data(int pType){
		byte[] getData = new byte[]{(byte) 0xA0,(byte) pType,(byte) 0x00};
		return sum_Check1(getData);
	}
	
	/**
	 * 删除数据
	 * @param pType
	 * 		0：删除脉率数据，1：删除血氧数据
	 * @return
	 */
	public static byte[] delete_Continuous_Data(int pType){
		byte[] getData = new byte[]{(byte) 0xA1,(byte) pType,(byte) 0x00};
		return sum_Check1(getData);
	}
	
	/**
	 * 请求脉率数据
	 * @param pType
	 * 		0：传输，127：终止
	 * @return
	 */
	public static byte[] get_Pulse_Rate(int pType){
		byte[] getData = new byte[]{(byte) 0xA2,(byte) pType,(byte) 0x00,(byte) 0x00,(byte) 0x00};
		return sum_Check1(getData);
	}
	
	/**
	 * 请求血氧数据
	 * @param pType
	 * 		0：传输，127：终止
	 * @return
	 */
	public static byte[] get_Pulse_Oxygen(int pType){
		byte[] getData = new byte[]{(byte) 0xA3,(byte) pType,(byte) 0x00,(byte) 0x00,(byte) 0x00};
		return sum_Check1(getData);
	}
	/**
	 * 获取校验码
	 * @param code 
	 * @param data 血氧数据
	 * @return
	 */
	public static byte[] device_Handshake_old(byte[] data)
	{
		byte[] code = new byte[8];
		byte[] md5_code;
		try {
			md5_code = MessageDigest.getInstance("MD5").digest(data);
			
			for (int i=0; i<8; ++i)
			{
				code[i] = (byte)((md5_code[i] ^ md5_code[i + 8]) | 0x80);
			}
			

			byte[] _emtrypt = new byte[] {(byte) 0x21,(byte) code[0],(byte) code[1],(byte) code[2],(byte) code[3],(byte) code[4],
					(byte) code[5],(byte) code[6],(byte) code[7],(byte)0x00};
			return sum_Check(_emtrypt);
		} catch (NoSuchAlgorithmException e) {
			
			e.printStackTrace();
			return null;
		}
	}
	
	// 旧版50f校验和
	public static byte[] sum_Check(byte[] pack) {
		int CHECK_SUM = 0;
		int _size = pack.length - 1;
		for (int i = 0; i < _size; i++) {
			CHECK_SUM += (pack[i] & 0xff);
		}
		byte _return = (byte) ((CHECK_SUM & 0x7f) | 0x80);
		pack[pack.length - 1] = _return;
		return pack;
	}
	// 新版50f检验和
	public static byte[] sum_Check1(byte[] pack) {
		int CHECK_SUM = 0;
		int _size = pack.length - 1;
		for (int i = 0; i < _size; i++) {
			CHECK_SUM += (pack[i] & 0xff);
		}
		byte _return = (byte) (CHECK_SUM & 0x7f);
		pack[pack.length - 1] = _return;
		return pack;
	}
	
}
